/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
* 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
* 				Marian Pohling, Lukas Twardon
*/
package de.unibi.airobots.bonsai.actuators;

import java.io.IOException;

import de.unibi.airobots.bonsai.core.Actuator;
import de.unibi.airobots.btl.data.PersonData;

/**
 * Interface for actuators controlling the HWA.
 * 
 * @author jwienke
 */
public interface HwaActuator extends Actuator {

    /**
     * Start looking for persons.
     * 
     * @throws IOException
     *             communication error
     */
    void startCamAction() throws IOException;

    /**
     * Focus the given person as long as it is known by the person tracker.
     * 
     * @param p
     *            person to focus
     * @throws IOException
     *             communication error
     */
    void focus(PersonData p) throws IOException;

    /**
     * Unfocus any person and start looking again.
     * 
     * @throws IOException
     *             communication error
     */
    void unfocus() throws IOException;

    /**
     * Stop controlling the camera.
     * 
     * @throws IOException
     *             communication error
     */
    void stopCamAction() throws IOException;

}
