/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
* 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
* 				Marian Pohling, Lukas Twardon
*/
package de.unibi.airobots.bonsai.actuators;

import java.io.IOException;

import net.sf.xcf.MethodNotFoundException;
import net.sf.xcf.UserException;
import de.unibi.airobots.bonsai.actuators.data.CameraPosition;
import de.unibi.airobots.bonsai.core.Actuator;

/**
 * Old interface to control a pan-tilt camera.
 * 
 * @author jwienke
 */
public interface OldCameraActuator extends Actuator {

    /**
     * Set the position of the Camera.
     * 
     * @param pan
     *            angle in radiant (0=forward, angles CCW)
     * @param tilt
     *            angle in radiant (0=forward, negative = down)
     * @throws IOException
     */
    void setPos(double pan, double tilt) throws IOException;

    /**
     * set the Zoom of the Camera.
     * 
     * @param zoom
     *            of the camera between 0 and 1 (0=wide, 1=tele)
     * @throws IOException
     */
    void setZoom(double zoom) throws IOException;

    /**
     * set the speed of pan movements.
     * 
     * @param panspeed
     *            of the cameras pan movement
     * @throws IOException
     */
    void setPanSpeed(double panspeed) throws IOException;

    /**
     * set the speed of tilt movements.
     * 
     * @param tiltspeed
     *            of the cameras tilt movement
     * @throws IOException
     */
    void setTiltSpeed(double tiltspeed) throws IOException;

    /**
     * set the zoom speed.
     * 
     * @param zoomspeed
     *            speed
     * @throws IOException
     */
    void setZoomSpeed(double zoomspeed) throws IOException;

    /**
     * Get the current zoom of the camera.
     * 
     * @return [0..1] 0 = wide, 1 = tele
     * @throws IOException
     *             communication error
     */
    float getZoomPos() throws IOException;

    CameraPosition getCameraPosition() throws IOException,
            MethodNotFoundException, UserException;

}
