/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
* 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
* 				Marian Pohling, Lukas Twardon
*/
package de.unibi.airobots.bonsai.actuators;

import java.util.HashMap;

import net.sf.xcf.transport.XOPData;

import de.unibi.airobots.bonsai.core.Actuator;
import de.unibi.airobots.btl.data.taskpattern.TaskOriginData;
import de.unibi.airobots.btl.data.taskpattern.TaskStateData;


/**
 * Actuator interface to control the task pattern communication between the task components.
 * @author arothert, vrichter, bcarlmey
 */
public interface TaskPatternActuator extends Actuator {
    
    /**
     * This method initializes a task. 
     * @param XPath either just the task name as XOPData or the whole task representation. 
     */
    void initiate(XOPData taskSpec);
    
    /**
     * simple state update of a task.
     * @param taskSpec the task specification as XOPDatat (might just be the task name). 
     * @param taskState TaskStateData to update the state of the status of the task.
     * @param taskOrigin TaskOriginData to update the origin of the status of the task.
     */
    void update(XOPData taskSpec, TaskStateData taskState, TaskOriginData taskOrigin);

    /**
     * attribute update including state update.
     * @param taskSpec the task specification as XOPDatat (might just be the task name).
     * @param attribute the name as String of the attribute which needs to be updated or inserted within the task.
     * @param value the value of the to be updated attribute as String. 
     * @param taskState TaskStateData representing the (new) state of the status of the task.
     * @param taskOrigin TaskOriginData representing the (new) origin of the status of the task.
     */
    void attributeUpdate(XOPData taskSpec, String attribute, String value, TaskStateData taskState, TaskOriginData taskOrigin);
    
    /**
     * * attribute update including state update.
     * @param task
     * @param mapOfAttributes HashMap representing the attribute (key) and the value of the attribute (value).
     * @param taskSpec the task specification as XOPDatat (might just be the task name).
     * @param mapOfAttributes HashMap<String, String> representing the attribute (key) and the value of the attribute (value).
     * @param taskState TaskStateData representing the (new) state of the status of the task.
     * @param taskOrigin TaskOriginData representing the (new) origin of the status of the task.
     */
    void attributeUpdate(XOPData taskSpec, HashMap<String,String> mapOfAttributes, TaskStateData taskState, TaskOriginData taskOrigin);

}


