/*
* This file is part of Bielefeld Sensor Actuator Interface (BonSAI).
*
* Copyright(c) Frederic Siepmann
* http://opensource.cit-ec.de/projects/bonsai
*
* This file may be licensed under the terms of of the
* GNU Lesser General Public License Version 3 (the ``LGPL''),
* or (at your option) any later version.
*
* Software distributed under the License is distributed
* on an ``AS IS'' basis, WITHOUT WARRANTY OF ANY KIND, either
* express or implied. See the LGPL for the specific language
* governing rights and limitations.
*
* You should have received a copy of the LGPL along with this
* program. If not, go to http://www.gnu.org/licenses/lgpl.html
* or write to the Free Software Foundation, Inc.,
* 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
*
* The development of this software was supported by the
* Excellence Cluster EXC 277 Cognitive Interaction Technology.
* The Excellence Cluster EXC 277 is a grant of the Deutsche
* Forschungsgemeinschaft (DFG) in the context of the German
* Excellence Initiative.
*
* Contributors: Leon Ziegler, Johannes Wienke, Norman Köster, Torben Töniges,
* 				Annika Rothert, Alexander Baumann, Marcel Dulisch, Maikel Linke,
* 				Marian Pohling, Lukas Twardon
*/
package de.unibi.airobots.bonsai.examples;

import java.awt.Component;
import java.awt.Container;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.io.IOException;
import java.net.URISyntaxException;

import javax.swing.JButton;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JTextField;

import de.unibi.airobots.bonsai.actuators.PoseActuatorTobi;
import de.unibi.airobots.bonsai.core.BonsaiManager;
import de.unibi.airobots.bonsai.core.BonsaiManager.ConfigurationException;
import de.unibi.airobots.bonsai.util.arm.ArmStatus;
import de.unibi.airobots.btl.data.KatanaPoseData;
import de.unibi.airobots.btl.data.PoseData;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

public class ArmTest extends JPanel implements ActionListener {

    /**
     * 
     */
    private static final long serialVersionUID = -8127787133332161066L;

    private static final Logger LOGGER = Logger.getLogger(ArmTest.class);

    private PoseActuatorTobi poseActuator;

    // private Sensor<PoseData> armCoordinates;

    private JLabel xLabel;
    private JLabel yLabel;
    private JLabel zLabel;

    private JLabel status;

    private JLabel gotoXLabel;
    private JLabel gotoYLabel;
    private JLabel gotoZLabel;
    private JLabel gotoPhiLabel;
    private JLabel gotoPsiLabel;
    private JLabel gotoThetaLabel;

    private JTextField executeXTextFild;
    private JTextField executeYTextFild;
    private JTextField executeZTextFild;

    private JTextField gotoXTextFild;
    private JTextField gotoYTextFild;
    private JTextField gotoZTextFild;
    private JTextField gotoPhiTextFild;
    private JTextField gotoPsiTextFild;
    private JTextField gotoThetaTextFild;

    public ArmTest() throws IllegalStateException, ConfigurationException,
            URISyntaxException, IOException {
        BasicConfigurator.configure();

        try {
            BonsaiManager.getInstance().configure(
                    "/vol/robocup/trunk/etc/bonsai_config/armTest.xml");

        } catch (IllegalStateException e) {
            e.printStackTrace();
        } catch (ConfigurationException e) {
            e.printStackTrace();
        }

        // armCoordinates =
        // BonsaiManager.getInstance().createSensor("ArmCoordinates",
        // PoseData.class);

        poseActuator =
                BonsaiManager.getInstance().createActuator("PoseActuatorTobi",
                        PoseActuatorTobi.class);

        this.guiSetup();
    }

    private void guiSetup() {
        JFrame gui = new JFrame("ArmTest for Biron");
        gui.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
        gui.setLocation(200, 200);

        xLabel = new JLabel("x:");
        yLabel = new JLabel("y:");
        zLabel = new JLabel("z:");

        status = new JLabel();

        executeXTextFild = new JTextField(2);
        executeYTextFild = new JTextField(2);
        executeZTextFild = new JTextField(2);

        JButton executeButton = new JButton("execute Strategy");
        executeButton.addActionListener(this);

        gotoXLabel = new JLabel("x:");
        gotoYLabel = new JLabel("y:");
        gotoZLabel = new JLabel("z:");
        gotoPhiLabel = new JLabel("Psi:");
        gotoPsiLabel = new JLabel("Phi:");
        gotoThetaLabel = new JLabel("Theta:");

        gotoXTextFild = new JTextField(2);
        gotoYTextFild = new JTextField(2);
        gotoZTextFild = new JTextField(2);
        gotoPhiTextFild = new JTextField(2);
        gotoPsiTextFild = new JTextField(2);
        gotoThetaTextFild = new JTextField(2);

        JButton goToNonLinButton = new JButton("goToNonLinear");
        goToNonLinButton.addActionListener(this);

        JButton goToLinButton = new JButton("goToLinear");
        goToLinButton.addActionListener(this);

        JButton closeForceButton = new JButton("closeByForce");
        closeForceButton.addActionListener(this);

        JButton openWhenTouchingButton = new JButton("openWhenTouching");
        openWhenTouchingButton.addActionListener(this);

        JButton getBoxButton = new JButton("getBox");
        getBoxButton.addActionListener(this);

        Container pane = gui.getContentPane();
        GridBagLayout gbl = new GridBagLayout();
        pane.setLayout(gbl);

        // cont, gbl, comp, x, y, width, height, weightx, weighty
        addComponent(pane, gbl, xLabel, 1, 0, 1, 1, 1.0, 0);
        addComponent(pane, gbl, executeXTextFild, 2, 0, 2, 1, 1.0, 0);
        addComponent(pane, gbl, yLabel, 4, 0, 1, 1, 1.0, 0);
        addComponent(pane, gbl, executeYTextFild, 5, 0, 2, 1, 1.0, 0);
        addComponent(pane, gbl, zLabel, 7, 0, 1, 1, 1.0, 0);
        addComponent(pane, gbl, executeZTextFild, 8, 0, 2, 1, 1.0, 0);

        addComponent(pane, gbl, executeButton, 10, 0, 2, 1, 1.0, 0);

        addComponent(pane, gbl, gotoXLabel, 1, 1, 1, 1, 1.0, 0);
        addComponent(pane, gbl, gotoXTextFild, 2, 1, 2, 1, 1.0, 0);
        addComponent(pane, gbl, gotoYLabel, 4, 1, 1, 1, 1.0, 0);
        addComponent(pane, gbl, gotoYTextFild, 5, 1, 2, 1, 1.0, 0);
        addComponent(pane, gbl, gotoZLabel, 7, 1, 1, 1, 1.0, 0);
        addComponent(pane, gbl, gotoZTextFild, 8, 1, 2, 1, 1.0, 0);

        addComponent(pane, gbl, gotoPhiLabel, 1, 2, 1, 1, 1.0, 0);
        addComponent(pane, gbl, gotoPhiTextFild, 2, 2, 2, 1, 1.0, 0);
        addComponent(pane, gbl, gotoPsiLabel, 4, 2, 1, 1, 1.0, 0);
        addComponent(pane, gbl, gotoPsiTextFild, 5, 2, 2, 1, 1.0, 0);
        addComponent(pane, gbl, gotoThetaLabel, 7, 2, 1, 1, 1.0, 0);
        addComponent(pane, gbl, gotoThetaTextFild, 8, 2, 2, 1, 1.0, 0);

        addComponent(pane, gbl, goToNonLinButton, 10, 1, 1, 2, 1.0, 0);
        addComponent(pane, gbl, goToLinButton, 11, 1, 1, 2, 1.0, 0);

        addComponent(pane, gbl, closeForceButton, 1, 5, 11, 1, 1.0, 0);
        addComponent(pane, gbl, openWhenTouchingButton, 1, 6, 11, 1, 1.0, 0);

        addComponent(pane, gbl, getBoxButton, 1, 7, 11, 1, 1.0, 0);

        addComponent(pane, gbl, status, 1, 8, 11, 1, 1.0, 0);

        gui.pack();
        gui.setVisible(true);
    }

    /**
     * A useful function to add all components into the GridBagLayout.
     */
    private static void addComponent(Container cont, GridBagLayout gbl,
            Component c, int x, int y, int width, int height, double weightx,
            double weighty) {
        GridBagConstraints gbc = new GridBagConstraints();
        gbc.fill = GridBagConstraints.BOTH;
        gbc.gridx = x;
        gbc.gridy = y;
        gbc.gridwidth = width;
        gbc.gridheight = height;
        gbc.weightx = weightx;
        gbc.weighty = weighty;
        gbl.setConstraints(c, gbc);
        cont.add(c);
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        if (e.getActionCommand().equals("execute Strategy")) {
            System.out.println("Executing Grasp strategy");

            try {
                PoseData pose = new PoseData();
                pose.setX(Double.parseDouble(executeXTextFild.getText()));
                pose.setY(Double.parseDouble(executeYTextFild.getText()));
                pose.setZ(Double.parseDouble(executeZTextFild.getText()));

                ArmStatus armStatus = poseActuator.executeStrategy(pose);

                System.out.println("Approach: "
                        + armStatus.isApproachPositionReached() + " Lift: "
                        + armStatus.isLiftPositionReached() + " Grasp: "
                        + armStatus.isGraspingObject() + " Carry: "
                        + armStatus.isCarryPositionReached());

            } catch (IOException e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
            }
        } else if (e.getActionCommand().equals("closeByForce")) {
            System.out.println("Closing by Force");

            try {
                poseActuator.closeGripperByForce();
            } catch (IOException e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
            }

        } else if (e.getActionCommand().equals("openWhenTouching")) {
            System.out.println("open when touching");

            try {
                poseActuator.openGripperWhenTouching(0);
            } catch (IOException e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
            }
        } else if (e.getActionCommand().equals("goToNonLinear")) {
            System.out.println("goto non linear");

            try {
                KatanaPoseData pose = new KatanaPoseData();
                pose.setX(Double.parseDouble(gotoXTextFild.getText()));
                pose.setY(Double.parseDouble(gotoYTextFild.getText()));
                pose.setZ(Double.parseDouble(gotoZTextFild.getText()));
                pose.setPhi(Double.parseDouble(gotoPhiTextFild.getText()));
                pose.setPsi(Double.parseDouble(gotoPsiTextFild.getText()));
                pose.setTheta(Double.parseDouble(gotoThetaTextFild.getText()));

                if (poseActuator.goTo(pose, false)) {
                    status.setText("Goal reached!");
                } else {
                    status.setText("Goal not reached!");
                }

            } catch (IOException e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
            }

        } else if (e.getActionCommand().equals("goToLinear")) {
            System.out.println("goto linear");

            try {
                KatanaPoseData pose = new KatanaPoseData();
                pose.setX(Double.parseDouble(gotoXTextFild.getText()));
                pose.setY(Double.parseDouble(gotoYTextFild.getText()));
                pose.setZ(Double.parseDouble(gotoZTextFild.getText()));
                pose.setPhi(Double.parseDouble(gotoPhiTextFild.getText()));
                pose.setPsi(Double.parseDouble(gotoPsiTextFild.getText()));
                pose.setTheta(Double.parseDouble(gotoThetaTextFild.getText()));

                if (poseActuator.goTo(pose, true)) {
                    status.setText("Goal reached!");
                } else {
                    status.setText("Goal not reached!");
                }
            } catch (IOException e1) {
                // TODO Auto-generated catch block
                e1.printStackTrace();
            }
        } else if (e.getActionCommand().equals("getBox")) {
            System.out.println("getBox");

            // PoseData data;
            // try {
            // data = armCoordinates.readLast();
            // status.setText("x: " + data.getX() + " | y: " + data.getY()
            // + " | z: " + data.getZ());
            //
            // } catch (IOException e1) {
            // // TODO Auto-generated catch block
            // e1.printStackTrace();
            // } catch (InterruptedException e1) {
            // // TODO Auto-generated catch block
            // e1.printStackTrace();
            // }

        }

    }

    /**
     * Startup main-method.
     * 
     * @param args
     *            none
     */
    public static void main(String[] args) {
        try {
            new ArmTest();
        } catch (Exception e) {
            e.printStackTrace();
            System.exit(1);
        }
    }
}
