package de.unibi.airobots.bonsai.xcf.configuration;

import java.util.Map;

import de.unibi.airobots.bonsai.core.CoreObjectFactory.ActuatorToConfigure;
import de.unibi.airobots.bonsai.sensors.MapAnnotationSensor;
import de.unibi.airobots.bonsai.xcf.sensors.XcfMapAnnotationSensor;
import de.unibi.airobots.btl.data.annotation.MapAnnotation;

/**
 * Configuration for {@link XcfMapAnnotationSensor}.
 * <p>
 * Available options:
 * <ul>
 * <li>{@value #OPTION_MEMORY_NAME} (required): Name of the active memory that
 * contains the information</li>
 * </ul>
 * 
 * @author vrichter
 */
public class ConfiguredXcfMapAnnotationSensor extends
        ConfiguredXcfSensor<MapAnnotation> {

    private String memoryName = "";

    /**
     * Name of the user option to specify the active memory
     */
    public static final String OPTION_MEMORY_NAME = "memoryName";

    /**
     * {@inheritDoc}
     * 
     * parsing the memory name.
     */
    @Override
    public void parseOptions(Map<String, String> options)
            throws IllegalArgumentException {

        if (!options.containsKey(OPTION_MEMORY_NAME)) {
            throw new IllegalArgumentException(
                    "Memory name missing with option " + OPTION_MEMORY_NAME);
        }

        memoryName = options.get(OPTION_MEMORY_NAME);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public MapAnnotationSensor createInstance() {
        return new XcfMapAnnotationSensor(memoryName);
    }

    /**
     * Checks if the sensor that can be produced is suitable for the requested
     * configuration.
     * 
     * @param sensor
     *            requested configuration
     * @return <code>true</code> if suitable
     */
    public static boolean isSuitableFor(ActuatorToConfigure actuator) {
        return actuator.getActuatorClass().equals(XcfMapAnnotationSensor.class);
    }

    /**
     * {@inheritDoc}
     */
    @Override
    public String[] getSubscribersToInitialize() {
        return new String[0];
    }
    
    /**
     * {@inheritDoc}
     */
    @Override
    public String[] getActiveMemoriesToInitialize() {
        return new String[] { memoryName };
    }
}
